"""

Virtual Site Root standard plugins

"""

from Products.PlugIns import PlugIn, PlugInGroup, defaultConstructors, genericAddForm, PlugInContainer

import Acquisition
import AccessControl.Role

from Globals import HTMLFile, MessageDialog, Persistent
from OFS.SimpleItem import Item
from Acquisition import Implicit, ImplicitAcquisitionWrapper
from ExtensionClass import Base
from string import split, strip
from ZPublisher import BeforeTraverse
import os,string, sys

from OFS.SimpleItem import SimpleItem


class VSRBasicPlugIn(PlugIn,SimpleItem):
    """ basic vsr plugin """
    __plugin_kind__="VSR PlugIn"

    def _handlePath(self,vsr,vsc,request): pass		    # handle a path request
    def _handleNoPath(self,vsr,vsc,request): pass	    # handle a no_path request


class SQLSessionPlugin(PlugIn,SimpleItem):
    """ SQL Session handling object """

    meta_type = 'VSR SQLSession PlugIn'
    __plugin_kind__="VSR PlugIn"

    #manage_options=({'label':'Edit', 'action':'manage_main'},)

    def __init__(self,id,title=''):
        ''' initialize the sql session object '''
	self.id=id
        self.title = title

    def _handlePath(self,vsr,vsc,request):
	""" handle a path """
	#  Session ID holen^M
	stack=request['TraversalRequestNameStack']
	sid=stack.pop()

	# test the session if it's valid or if we might have a path without a session id
	try:
	    self.get_session(vsr,vsc)(session_id=sid, noCookie=1, validSession=1)
	except:
	    # save old sid which actually wasn't one
	    old_sid=sid
	    # no valid session id: generate one
	    sid=self.create_session(vsr,vsc,request)

	    # now do a redirect: reverse the stack to create the URL
	    new_stack=stack+[old_sid,sid]
	    new_stack.reverse()
	    rest_stack=string.joinfields(new_stack,"/")
	    newurl=request['URL1']+"/"+rest_stack
	    if request.has_key('QUERY_STRING'):
		qs=request['QUERY_STRING']
		if len(qs)>0:
		    newurl=newurl+"?"+qs
	    raise 'Redirect',newurl

	# normal processing: just set the session id
	request.set('SessionID',sid)
	request.setVirtualRoot([sid])

    def _handleNoPath(self,vsr,vsc,request):
	""" handle the situation when there is no path """
	sid=self.create_session(vsr,vsc,request)
	raise 'Redirect',request['URL1']+"/"+sid+"/"

    def get_session(self,vsr,vsc):
	""" get the session object 

	first we need to get the virtual site folder and then we
	search inside this for the valid session object.
	The name of the virtual site folder is stored in the
	"folder" property in the virtual site config object
	"""

	# (root/virtual_site_config/plugin)

	vfolder=getattr(vsr,vsc.folder)
	return vfolder.Session

    def create_session(self,vsr,vsc,request):
	""" create a new session """
	sid=self.get_session(vsr,vsc).newSession(request,noCookie=1)
	request.set('SessionID',sid)
	request.setVirtualRoot([sid])
	return sid


def initialize(context):
    context.registerPlugInClass(
        SQLSessionPlugin,
        permission = 'Add SQL Session Support',
        constructors = defaultConstructors(SQLSessionPlugin,globals())
        #icon = 'www/specialist.gif'
    )


