"""SiteRoot module

Defines the Traverser base class and SiteRoot class
"""

import Acquisition
import AccessControl.Role

from Globals import HTMLFile, MessageDialog, Persistent
from OFS.SimpleItem import Item
from Acquisition import Implicit, ImplicitAcquisitionWrapper
from ExtensionClass import Base
from string import split, strip
from ZPublisher import BeforeTraverse
import os,string

from Products.PlugIns import PlugIn, PlugInGroup, defaultConstructors, genericAddForm, PlugInContainer

# use the SiteRoot Traverser (hey, re-use!! ;-)
from Products.SiteAccess.SiteRoot import Traverser

class VirtualSiteRoot(Traverser, Implicit):
    """ a virtual site root product """
    
    meta_type = 'Virtual Site Root'
    title = ''
    priority = 50
    strict=None		# if set to 1 then we will throw an exception if no config object is found
			# if set to None we will ignore this and do nothing.

    def manage_main(self):
	""" just redirect """
	raise "Redirect", self.aq_parent.aq_parent.absolute_url()+"/manage_main"

    manage=manage_main

    def __init__(self, id, title=''):
        '''Title'''
	self.id=id
        self.title = strip(title)

    def manage_edit(self, title, base, path, REQUEST=None):
        '''Set the title, base, and path'''
        self.__init__(title, base, path)
        if REQUEST:
          return MessageDialog(title='SiteRoot changed.',
            message='The title is now "%s"<br>'
                    'The base is now "%s"<br>'
                    'The path is now "%s"<br>' % (title, base, path),
            action='%s/manage_main' % REQUEST['URL1'])

    def __call__(self, client, request, response=None):
        '''Traversing

	TODO: 
	
	    - check for FTP!
	    - create plugin mechanism
	
	'''

	# get the configuration object
	id=self.id
	try:
	    config=getattr(self.aq_parent.aq_parent,id)
	    if config.meta_type!="Virtual Site Config":
		raise "Virtual Site Error","can't find Configuration object '%s'" %id
	except:
	    if self.strict:
		raise "Virtual Site Error","can't find Configuration object '%s'" %id
	    else:
		return

	# check for ftp
	if request.has_key('SERVER_SOFTWARE'):
	    if request['SERVER_SOFTWARE'][:3]=="FTP":
		return

	# get what we need to traverse
	stack=request['TraversalRequestNameStack']
	if stack:
	    # check for misc_ and don't redirect!
	    if stack[-1]=="misc_":
		return

	    # check for some management url
	    stacklist="/"+string.joinfields(stack,"/")
	    if string.find(stacklist,"/manage")!=-1:
		request['SCRIPT_NAME']=config.management_path_prefix
		request['SERVER_URL']=config.management_prefix
		request._resetURLS()
		request.setVirtualRoot(config.management_path_prefix)

		return

	    else: # no management, but a normal path
		# in any case we need to set the public prefix
		request.setVirtualRoot(config.public_path_prefix)
		request['SERVER_URL']=config.public_prefix
		request._resetURLS()

		# call plugins if available
		for plugin in config.handlerList:
		    plugin._handlePath(self,config,request)

	else:
	    # in any case we need to set the public prefix
	    request['SERVER_URL']=config.public_prefix
	    request.setVirtualRoot(config.public_path_prefix)
	    request._resetURLS()

	    # no stack available. This might mean something to a plugin, so call them here
	    for plugin in config.handlerList:
		plugin._handleNoPath(self,config,request)

    def get_size(self):
        '''Make FTP happy'''
        return 0

class VirtualSiteConfig(PlugInContainer):
    """ virtual site configurator object  

    This object is used in conjunction with a VirtualSiteRoot object and
    will store the neccessary information for the latter.
    
    """

    meta_type = 'Virtual Site Config'
    icon="misc_/VirtualSiteRoot/SiteRoot.gif"

    HandlersGroup = PlugInGroup('HandlersGroup',['VSR PlugIn'],attr='handlerList',title='Handlers')
    handlerList=()
    __plugin_groups__=(HandlersGroup,)

    manage_options_left=({'label':'Edit', 'action':'manage_main', 'help': ('VirtualSiteRoot','vsiteroot.stx')},)

    manage = manage_main = HTMLFile('www/edit', globals())

    def __init__(self, id, public_prefix, management_prefix, public_path_prefix='/', management_path_prefix='/', title='',folder=''):
        ''' initialize the configuration object '''
	self.id=id
        self.title = strip(title)
	self.public_prefix=public_prefix
	self.management_prefix=management_prefix
	self.public_path_prefix=public_path_prefix
	self.management_path_prefix=management_path_prefix
	self.folder=folder

    def manage_edit(self,title='',public_prefix=None,management_prefix=None, public_path_prefix='/', management_path_prefix='/', REQUEST=None):
	""" change config properties  """
	if public_prefix: self.public_prefix=public_prefix
	if management_prefix: self.management_prefix=management_prefix
	self.public_path_prefix=public_path_prefix
	self.management_path_prefix=management_path_prefix
	self.title=title
	if REQUEST:
	    return self.manage_main(self,REQUEST)



def manage_addVirtualSiteRoot(self, id, folder, title='', 
			    public_prefix=None, 
			    management_prefix=None, 
			    public_path_prefix='/', 
			    management_path_prefix='/', 
			    REQUEST=None, **ignored):
    """ add a VirtualSiteRoot and a VirtualSiteConfigurator """

    if not public_prefix:
	raise "BadRequest", "no public prefix is given"

    if not management_prefix:
	raise "BadRequest", "no management prefix is given"

    # get the folder object
    of=getattr(self,folder)
    # create the VRoot object
    vsr=VirtualSiteRoot(id,title)
    vsr.manage_addToContainer(of.this())

    # create the configurator object
    vsc=VirtualSiteConfig(id,
	    public_prefix,
	    management_prefix,
	    public_path_prefix, 
	    management_path_prefix, title,folder=folder)
    self._setObject(id,vsc)

    if REQUEST:
	return self.manage_main(self,REQUEST,update_menu=1)

constructors = (
  ('manage_addForm', HTMLFile('www/SiteRootAdd', globals())),
  ('manage_add', manage_addVirtualSiteRoot),
)
